/*

  This is a simple tool for doing some image processing operations.
  It simply calls the right programs to do the actual work.

  Written by Mark Overmars

*/

#include <stdio.h>
#include <string.h>
#include <gl/device.h>
#include "forms.h"
#include "iforms.h"
#include "idisp.h"

char tmp_str[256];

/**** General Tool *********/

float get_value(char str[], float start, float min, float max,
			 float smallstep,float largestep, int prec)
{
  char ttt[32];
  float val = start;
  FL_OBJECT *obj;
  if (current_file_name() == NULL) return;
  fl_deactivate_form(main_form);
  fl_set_object_label(value_name,str);
  sprintf(ttt,"%.*f",prec,val);
  fl_set_object_label(value_value,ttt);
  fl_show_form(value_form,FL_PLACE_MOUSE,FALSE,NULL);
  do
  {
    sprintf(ttt,"%.*f",prec,val);
    fl_set_object_label(value_value,ttt);
    obj = fl_do_forms();
    if (obj == value_lleft) val -= largestep;
    if (obj == value_left) val -= smallstep;
    if (obj == value_right) val += smallstep;
    if (obj == value_rright) val += largestep;
    if (val<min) val = min;
    if (val>max) val = max;
  }
  while (obj != value_ready);
  fl_hide_form(value_form);
  fl_activate_form(main_form);
  return val;
}

void do_action(char startstr[], char endstr[])
{
  if (current_file_name() == NULL) return;
  fl_deactivate_form(main_form);
  fl_redraw_form(main_form);
  fl_set_object_label(title_obj,"- THINKING -");
  new_file_name();
  sprintf(tmp_str,"%s %s %s %s",
		 startstr, previous_file_name(), current_file_name(), endstr);
  system(tmp_str);
  set_image_file();
  fl_set_object_label(title_obj,"Image Processing Tool");
  fl_activate_form(main_form);
}

/********* Callback routines *********/

/****** Files ******/

void load_cb(FL_OBJECT *obj, long arg)
{
  char *ttt;
  fl_deactivate_form(main_form);
  ttt = fl_show_file_selector("Image File to Load","","*.rgb","");
  if (ttt != NULL && ttt[0] != NULL)
  {
    sprintf(tmp_str,"/bin/cp %s %s",ttt,new_file_name());
    if (! system(tmp_str)) set_image_file(); else last_file_name();
  }
  fl_activate_form(main_form);
}

void save_cb(FL_OBJECT *obj, long arg)
{
  char *ttt;
  if (current_file_name() == NULL) return;
  fl_deactivate_form(main_form);
  ttt = fl_show_file_selector("Image File to Save To","","*.rgb","");
  if (ttt != NULL && ttt[0] != NULL)
  { 
    sprintf(tmp_str,"/bin/cp %s %s",current_file_name(), ttt);
    system(tmp_str);
  }
  fl_activate_form(main_form);
}

void copy_cb(FL_OBJECT *obj, long arg)
{
  fl_deactivate_form(main_form);
  if (current_file_name() != NULL)
    {sprintf(tmp_str,"ipaste %s", current_file_name()); system(tmp_str);}
  fl_activate_form(main_form);
}

void last_cb(FL_OBJECT *obj, long arg)
  { last_file_name(); set_image_file(); }

/****** Operations ******/

void zoom_cb(FL_OBJECT *obj, long arg)
{
  float val = get_value("Give Zoom Factor.",1.0,0.1,100.0,0.1,1.0,1);
  char ttt[128];
  sprintf(ttt,"%f %f", val, val);
  do_action("/usr/sbin/izoom",ttt);
}

void zoomto_cb(FL_OBJECT *obj, long arg)
{
  float val1 = get_value("Give Width.",100.0,1.0,1280.0,1.0,20.0,0);
  float val2 = get_value("Give Height.",val1,1.0,1280.0,1.0,20.0,0);
  int w,h;
  char ttt[128];
  get_image_size(&w,&h);
  if (w == -1) return;
  val1 = val1 / w;
  val2 = val2 / h;
  sprintf(ttt,"%f %f", val1, val2);
  do_action("/usr/sbin/izoom",ttt);
}

void flip_cb(FL_OBJECT *obj, long arg)
{
  FL_OBJECT *obj;
  fl_deactivate_form(main_form);
  fl_show_form(flip_form,FL_PLACE_MOUSE,FALSE,NULL);
  do obj = fl_do_forms(); while (obj == FL_EVENT);
  fl_hide_form(flip_form);
  fl_activate_form(main_form);
  if (obj == flip_x  ) do_action("/usr/sbin/iflip","x");
  if (obj == flip_y  ) do_action("/usr/sbin/iflip","y");
  if (obj == flip_xy ) do_action("/usr/sbin/iflip","xy");
  if (obj == flip_yx ) do_action("/usr/sbin/iflip","yx");
  if (obj == flip_90 ) do_action("/usr/sbin/iflip","90");
  if (obj == flip_180) do_action("/usr/sbin/iflip","180");
  if (obj == flip_270) do_action("/usr/sbin/iflip","270");
}

void addframe_cb(FL_OBJECT *obj, long arg)
{
  do_action("/usr/sbin/addframe","");
}

void postcard_cb(FL_OBJECT *obj, long arg)
{
  do_action("/usr/sbin/postcard","");
}

void quantify_cb(FL_OBJECT *obj, long arg)
{
  float val = get_value("Number of Levels?",16.0,0.0,256.0,1.0,10.0,0);
  char ttt[128];
  sprintf(ttt,"%f", val);
  do_action("/usr/sbin/quant",ttt);
}

void expand_cb(FL_OBJECT *obj, long arg)
{
  do_action("/usr/sbin/imgexp","");
}

void hist_cb(FL_OBJECT *obj, long arg)
{
  if (fl_show_question("Add Random Noise?","",""))
    do_action("/usr/sbin/histeq","-r");
  else
    do_action("/usr/sbin/histeq","");
}

void blur_cb(FL_OBJECT *obj, long arg)
{
  float val = get_value("Size of blur image?",50.0,1.0,1000.0,1.0,10.0,0);
  char ttt[128];
  sprintf(ttt,"%f %f", val, val);
  do_action("/usr/sbin/blur",ttt);
}

void gamma_cb(FL_OBJECT *obj, long arg)
{
  float val = get_value("Give Gamma Value",1.0,0.0,100.0,0.05,0.5,2);
  char ttt[128];
  sprintf(ttt,"%f", val);
  do_action("/usr/sbin/gammawarp",ttt);
}

void saturate_cb(FL_OBJECT *obj, long arg)
{
  float val = get_value("Give Saturation Level",1.0,0.0,100.0,0.01,0.1,2);
  char ttt[128];
  sprintf(ttt,"%f", val);
  do_action("/usr/sbin/saturate",ttt);
}

void blackwhite_cb(FL_OBJECT *obj, long arg)
{
  do_action("/usr/sbin/saturate","0.0");
}

void abs_cb(FL_OBJECT *obj, long arg)
{
  do_action("/usr/sbin/abs","");
}

void invert_cb(FL_OBJECT *obj, long arg)
{
  do_action("/usr/sbin/invert","");
}

void noblack_cb(FL_OBJECT *obj, long arg)
{
  do_action("/usr/sbin/noblack","");
}

void halftone_cb(FL_OBJECT *obj, long arg)
{
  float val = get_value("Give Template Size",8.0,1.0,256.0,1.0,4.0,0);
  char ttt[128];
  sprintf(ttt,"dotgen /usr/tmp/dotgenim.tmp %f 1 1", val);
  system(ttt);
  do_action("/usr/sbin/thresh","/usr/tmp/dotgenim.tmp");
}

void dither_cb(FL_OBJECT *obj, long arg)
{
  system("greyscale /usr/tmp/dotgenim.tmp 8 8 18");
  do_action("/usr/sbin/thresh","/usr/tmp/dotgenim.tmp");
}

/****** Others ******/

void quit_cb(FL_OBJECT *obj, long arg)
{
  system("/bin/rm -f /usr/tmp/tmpimg*.rgb");
  exit(0);
}

void print_cb(FL_OBJECT *obj, long arg)
{
  FILE *fn;
  char str[512], str2[512];
  int ch, i=0, j=0;
  fl_deactivate_form(main_form);
  strcpy(str,fl_get_input(print_print));
  if (str[0] == NULL)
  {
    fn = fopen("printcommand","r");
    if (fn != (FILE *) NULL)
    {
      while ((ch = getc(fn)) != EOF && ch != '\n') str[i++] = ch;
      fclose(fn);
      str[i] = NULL;
      fl_set_input(print_print,str);
    }
  }
  fl_show_form(print_form,FL_PLACE_MOUSE,FALSE,NULL);
  do obj = fl_do_forms(); while (obj != print_print);
  fl_hide_form(print_form);
  fl_activate_form(main_form);
  strcpy(str,fl_get_input(print_print));
  if (str[0] == NULL) return;
  i = 0; j = 0;
  while (str[i] != NULL && str[i] != '#') str2[j++] = str[i++];
  str2[j] = NULL;
  if (str[i] != NULL) i++;
  strcat(str2,current_file_name());
  j = strlen(str2);
  while (str[i] != NULL) str2[j++] = str[i++];
  str2[j] = NULL;
  system(str2);
}

void snap_cb(FL_OBJECT *obj, long arg)
{
  fl_deactivate_form(main_form);
  system("/usr/sbin/snapshot &");
  fl_hide_object(obj);
  fl_activate_form(main_form);
}

/********************* Main program ******************/

main()
{
  FL_OBJECT *obj;
  long dev;
  short val;
  create_the_forms();
  fl_show_form(main_form,FL_PLACE_SIZE,TRUE,"Control");
  load_cb(NULL,0);
  while (1)
  {
    obj = fl_do_forms();
    if (obj == FL_EVENT)
    {
      dev = fl_qread(&val);
      if (dev == REDRAW) redraw_image();
    }
  }
}
